#pragma once

#include "FBullCowGame.h"
#include <map>
#include <cstdlib> 
#include <ctime> 


// to make syntax Unreal friendly
#define TMap std::map
using int32 = int;
using FString = std::string;


FbullCowGame::FbullCowGame() // default constructor
{
	reset();
}

int32 FbullCowGame::GetCurrentTry() const { return MyCurrentTry; }
int32 FbullCowGame::GetHiddenWordLength() const{ return MyHiddenWord.length(); }
bool FbullCowGame::IsGameWon() const{ return bGameIsWon; }

int32 FbullCowGame::GetMaxTries() const 
{
	TMap <int32, int32> WordLengthToMaxTries{ {3,5}, {4,8}, {5,10}, {6,16}, {7,20} };
	return WordLengthToMaxTries[MyHiddenWord.length()];
}

FString FbullCowGame::SetHiddenWord() const
{
	FString Words[] = { "cat", "map", "long", "kind", "plane", "action", "machine" };
	srand(time(NULL));
	return Words[(rand() % 7)];
}

void FbullCowGame::reset()
{
	const FString HIDDEN_WORD = SetHiddenWord(); // this MUST be an isogram

	MyHiddenWord = HIDDEN_WORD;
	MyCurrentTry = 1;
	bGameIsWon = false;
	return;
}


EGuessStatus FbullCowGame::CheckGuessValidity(FString Guess) const
{
	if (!IsIsogram(Guess)) //if the guess isn't an isogram
	{
		return EGuessStatus::Not_Isogram;
	}
	else if (!IsLowercase(Guess)) //if the guess isn't all lowercase
	{
		return EGuessStatus::Not_Lowercase;
	}
	else if (Guess.length() != GetHiddenWordLength()) //if the guess length is wrong
	{
		return EGuessStatus::Wrong_Length;
	}
	else 
	{
		return EGuessStatus::OK;
	}
}

// recieves a VALID guess, incriments truns and returns count
FBullCowCount FbullCowGame::SubmitValidGuess(FString Guess)
{
	MyCurrentTry++;
	FBullCowCount BullCowCount;
	int32 WordLength = MyHiddenWord.length();// assume same length as guess

	// loop through all letters in the hiddenword(MHWChar == MyHiddenWordCharacter)
	for (int32 MYHChar = 0; MYHChar < WordLength; MYHChar++)
	{
		// compare letters against the guess(Gchar = GuessCharacter)
		for (int32 Gchar = 0; Gchar < WordLength; Gchar++)
		{
			// if they match then
			if (Guess[Gchar] == MyHiddenWord[MYHChar])
			{
				if (MYHChar == Gchar)// if they're in the same place
				{
					BullCowCount.Bulls++;// incriment bulls
				}	
				else
				{
					BullCowCount.Cows++;// must be a cow
				}
			}
		}
	}
	if (BullCowCount.Bulls == WordLength)
	{
		bGameIsWon = true;
	}
	else
	{
		bGameIsWon = false;
	}
	return BullCowCount; 
}

bool FbullCowGame::IsIsogram(FString Word) const
{
	// treat 0 or 1 letter words as isogram
	if (Word.length() <= 1)
	{
		return true;
	}
	TMap <char, bool> LetterSeen; // setup map
	for (auto Letter : Word) //loop through all the letter in the word
	{
		Letter = tolower(Letter); // handle mixed case
		if (LetterSeen[Letter]) // if the letter is in the map
		{
			return false; // not Isogram
		}
		else // otherwise
		{
			LetterSeen[Letter] = true;
		}
			// add the letter to the map as seen
	}
	return true;
}

bool FbullCowGame::IsLowercase(FString Word) const
{
	if (Word.length() <= 1)
	{
		return true;
	}
	for (auto Letter : Word)
	{
		if (!islower(Letter))
		{
			return false;
		}
	}
	return true;
}
